package kr.co.vividnext.sodalive.live.tag

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.tag.QLiveTag.liveTag
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface LiveTagRepository : JpaRepository<LiveTag, Long>, LiveTagQueryRepository {
    fun findByTag(it: String): LiveTag?
}

interface LiveTagQueryRepository {
    fun getTags(role: MemberRole, isAdult: Boolean, cloudFrontHost: String): List<GetLiveTagResponse>
}

@Repository
class LiveTagQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : LiveTagQueryRepository {
    override fun getTags(role: MemberRole, isAdult: Boolean, cloudFrontHost: String): List<GetLiveTagResponse> {
        var where = liveTag.isActive.isTrue

        if (role != MemberRole.ADMIN && !isAdult) {
            where = where.and(liveTag.tag.notIn("음담패설", "EDPS"))
        }

        return queryFactory
            .select(
                QGetLiveTagResponse(
                    liveTag.id,
                    liveTag.tag,
                    liveTag.image.prepend("/").prepend(cloudFrontHost)
                )
            )
            .from(liveTag)
            .where(where)
            .orderBy(liveTag.orders.asc())
            .fetch()
    }
}
