package kr.co.vividnext.sodalive.member.tag

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.tag.QCreatorTag.creatorTag
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface MemberTagRepository : JpaRepository<CreatorTag, Long>, MemberTagQueryRepository {
    fun findByTag(tag: String): CreatorTag?
}

interface MemberTagQueryRepository {
    fun getTags(): List<GetMemberTagResponse>
}

class MemberTagQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : MemberTagQueryRepository {
    override fun getTags(): List<GetMemberTagResponse> {
        return queryFactory
            .select(
                QGetMemberTagResponse(
                    creatorTag.id,
                    creatorTag.tag,
                    creatorTag.image.prepend("/").prepend(cloudFrontHost)
                )
            )
            .from(creatorTag)
            .where(creatorTag.isActive.eq(true))
            .orderBy(creatorTag.orders.asc())
            .fetch()
    }
}
